//----- < SMHWCfg.h > --------------------------------------------------
#ifndef SMHWCFG_INCD
#define SMHWCFG_INCD

#include "..\Common.h"

/***************************************************************************
SmartMedia Controller Definition
***************************************************************************/
/* I/O Mode Address */
//#define DATA       (SMHostAddr+0x60)     // R/W Data   Reg
//#define STATUS     (SMHostAddr+0x62)     // R/- Status Reg
//#define MODE       (SMHostAddr+0x62)     // -/W Mode   Reg
//#define INTSTS     (SMHostAddr+0x64)     // 
//#define INTMSK     (SMHostAddr+0x66)     // 
//#define ENE        (SMHostAddr+0x67)     // EnE Define Reg

#define SM_REG_DATA        (SMHostAddr+0x00)     // SM Data Reg
#define SM_REG_STATUS      (SMHostAddr+0x04)     // SM Status Reg (R/-)
#define SM_REG_MODE        (SMHostAddr+0x04)     // SM Mode Reg   (-/W)
#define SM_REG_INT_STATUS  (SMHostAddr+0x05)     // SM Interrupt Status Reg
#define SM_REG_INT_MASK    (SMHostAddr+0x06)     // SM Interrupt Mask Reg
#define SM_REG_FIFO_STATUS (SMHostAddr+0x09)     // SM FIFO Status Reg
#define SM_REG_CTRL_1      (SMHostAddr+0x0E)     // SM Command Control Reg 1
#define SM_REG_CTRL_2      (SMHostAddr+0x0F)     // SM Command Control Reg 2
#define SM_REG_CTRL_3      (SMHostAddr+0x10)     // SM Command Control Reg 3
#define SM_REG_CTRL_4      (SMHostAddr+0x11)     // SM Command Control Reg 4
#define SM_REG_CTRL_5      (SMHostAddr+0x12)     // SM Command Control Reg 5
#define SM_REG_RESET_CTRL  (SMHostAddr+0x30)     // SM Reset/LED Control Reg 

#define SM_DMA_ADDR_REG         (SMHostAddr+0x40)     // SM DMA System Address Reg
#define SM_DMA_DATA_CTRL        (SMHostAddr+0x44)     // SM DMA Transfer Control Reg
#define SM_DMA_INT_STATUS       (SMHostAddr+0x45)     // SM DMA Status Reg
#define SM_DMA_INT_CTRL         (SMHostAddr+0x46)     // SM DMA Interrupt Control Reg
#define SM_DMA_CTRL             (SMHostAddr+0x47)     // SM DMA Control Reg
#define SM_DMA_IO_PORT_REG      (SMHostAddr+0x48)     // SM DMA I/O Port Address Reg
#define SM_DMA_BYTE_COUNT_REG   (SMHostAddr+0x4C)     // SM DMA Transfer Byte Count Reg


// (Offset: 04h) SM Status Reg 
/* Controller Status Reg (Read Only) */
#define STS_BUSY   0x80
#define STS_VCC    0x10
#define STS_SCHG   0x08
#define STS_CENB   0x04
#define STS_WP     0x01

// (Offset: 04h) SM Mode Reg 
/* Controller Mode Reg (Write Only) */
/*=======================================*/
#define STANDBY    0x00 /* ---0 --00 */
#define WR_DATA    0x10 /* ---1 --00 */
#define WR_CMD     0x11 /* ---1 --01 */
#define WR_ADR     0x12 /* ---1 --10 */
/*=======================================*/
#define ENB_WP     0x00 /* 0--- ---- */
#define DIS_WP     0x80 /* 1--- ---- */
/*=======================================*/
#define LED_OFF    0x00 /* ---- x0-- */
#define LED_ON     0x04 /* ---- x1-- */
/*=======================================*/
#define PW_OFF     0x08 /* ---- 10-- */
#define PW_ON      0x0C /* ---- 11-- */
/*=======================================*/
#define ECC_RESET  0x70 /* -111 ---- */
#define ECC_START  0x30 /* -011 ---- */
#define ECC_STOP   0x10 /* -001 ---- */
#define ECC_READ   0x50 /* -101 ---- */
/*=======================================*/

// (Offset: 08h) SM FIFO Control Reg 
#define SM_APB32_MASK           0x80    /*D[7]*/
#define SM_APB08_MASK           0x00
#define SM_DMAEN_MASK           0x20    /*D[5]*/
#define SM_DMADIS_MASK          0x00
#define SM_DMA_DOWNSTREAM_MASK  0x10    /*D[4]*/
#define SM_DMA_UPSTREAM_MASK    0x00
#define SM_FIFOSHLDVLU_4_MASK   0x00    /*D[1:0]*/
#define SM_FIFOSHLDVLU_8_MASK   0x01
#define SM_FIFOSHLDVLU_12_MASK  0x02

// (Offset: 09h) SM FIFO Status Reg 
#define SM_FIFOFULL_BIT         0x80
#define SM_FIFOEMPTY_BIT        0x40
#define SM_FIFOSHLD_INDICATOR   0X20
#define SM_FIFOSPACEIDCTR_MASK  0x0F

#define bSMFIFOOverShldForRd    (READ_PORT_BYTE(SM_FIFO_STAT) & SM_FIFOSHLD_INDICATOR)
#define bSMFIFOUndrShldForWr    (READ_PORT_BYTE(SM_FIFO_STAT) & SM_FIFOSHLD_INDICATOR)
#define bSMFIFOFULL             (READ_PORT_BYTE(SM_FIFO_STAT) & SM_FIFOFULL_BIT)
#define bSMFIFOEMPTY            (READ_PORT_BYTE(SM_FIFO_STAT) & SM_FIFOEMPTY_BIT)

// (Offset: 10h) SM Command Control Reg 3
#define SM_MEMSIZE_64MBPLUS   0x80
#define SM_PAGSIZE_512_BYTE   0x40
#define SM_WRECC_AT_SPACEDATA 0x20
#define SM_SOFTRESET_BIT      0x10
#define SM_BYTECNTHI_MASK     0x03

// (Offset: 12h) SM Command Control Reg 5
#define ENABLE_OB     0x80
#define ENABLE_3V     0x01
#define ENABLE_5V     0x02

#define _Wait_D_Timer(time)     EDelay(time*100)    // 1 unit = 0.1 ms

#define SM_FIFO_CTRL            *(SMHostAddr + 0x08)
#define SM_FIFO_STAT            *(SMHostAddr + 0x09)
#define SM_STARTADDR_LSB        *(SMHostAddr + 0x0A)
#define SM_STARTADDR_IISB       *(SMHostAddr + 0x0B)
#define SM_STARTADDR_IIISB      *(SMHostAddr + 0x0C)
#define SM_STARTADDR_MSB        *(SMHostAddr + 0x0D)
#define SM_CMD_CTRL1            *(SMHostAddr + 0x0E)
#define SM_BYTECNT_LO           *(SMHostAddr + 0x0F)
#define SM_BYTECNT_HI           *(SMHostAddr + 0x10)
#define SM_CMD_CTRL3            *(SMHostAddr + 0x10)

#define DMA_STARTADDRESS        *(SMHostAddr + 0x40)
#define DMA_XFERCTRL            *(SMHostAddr + 0x44)
#define DMA_XFER_BYCNT_MINUS1   *(SMHostAddr + 0x4C)

/***************************************************************************/
static BYTE LED = LED_OFF;

/***************************************************************************/
#define _Hw_D_InData()       READ_PORT_BYTE(SM_REG_DATA)
#define _Hw_D_OutData(a)     WRITE_PORT_BYTE(SM_REG_DATA, a)
/*****/
#define _Hw_D_SetRdData()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_DATA   | ENB_WP | LED))
#define _Hw_D_SetRdCmd()     WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_CMD    | ENB_WP | LED))
#define _Hw_D_SetRdAddr()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_ADR    | ENB_WP | LED))
#define _Hw_D_SetRdStandby() WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | ENB_WP | LED))
/*---*/
#define _Hw_D_SetWrData()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_DATA   | DIS_WP | LED))
#define _Hw_D_SetWrCmd()     WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_CMD    | DIS_WP | LED))
#define _Hw_D_SetWrAddr()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(WR_ADR    | DIS_WP | LED))
#define _Hw_D_SetWrStandby() WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | DIS_WP | LED))
/*****/
#define _Hw_D_VccOn()        WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | ENB_WP | PW_ON ))
#define _Hw_D_VccOff()       WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | ENB_WP | PW_OFF))
/*****/
#define _Hw_D_LedOn()        WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | ENB_WP | (LED=LED_ON )))
#define _Hw_D_LedOff()       WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(STANDBY   | ENB_WP | (LED=LED_OFF)))
/*****/
#define _Hw_D_EccRdReset()   WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_RESET | ENB_WP | LED))
#define _Hw_D_EccRdStart()   WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_START | ENB_WP | LED))
#define _Hw_D_EccRdStop()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_STOP  | ENB_WP | LED))
#define _Hw_D_EccRdRead()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_READ  | ENB_WP | LED))
/*---*/
#define _Hw_D_EccWrReset()   WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_RESET | DIS_WP | LED))
#define _Hw_D_EccWrStart()   WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_START | DIS_WP | LED))
#define _Hw_D_EccWrStop()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_STOP  | DIS_WP | LED))
#define _Hw_D_EccWrRead()    WRITE_PORT_BYTE(SM_REG_MODE, (BYTE)(ECC_READ  | DIS_WP | LED))
/*****/
#define _Hw_D_ChkCardIn()    (READ_PORT_BYTE(SM_REG_STATUS) & (STS_CENB))
#define _Hw_D_ChkStatus()    (READ_PORT_BYTE(SM_REG_STATUS) & (STS_SCHG))
#define _Hw_D_ChkWP()        (READ_PORT_BYTE(SM_REG_STATUS) & (STS_WP))
#define _Hw_D_ChkPower()     (READ_PORT_BYTE(SM_REG_STATUS) & (STS_VCC))
#define _Hw_D_ChkBusy()      (READ_PORT_BYTE(SM_REG_STATUS) & (STS_BUSY))
//#define _Hw_D_ChkModel()     (READ_PORT_BYTE(SM_REG_STATUS) & (STS_MODEL))

/*****/
#define _Hw_D_EnableVcc3VOn()   WRITE_PORT_BYTE(SM_REG_CTRL_5,  (BYTE)(ENABLE_3V))
#define _Hw_D_EnableVcc5VOn()   WRITE_PORT_BYTE(SM_REG_CTRL_5,  (BYTE)(ENABLE_5V))
#define _Hw_D_EnableOB()        WRITE_PORT_BYTE(SM_REG_CTRL_5,  (BYTE)(ENABLE_OB | ENABLE_5V | ENABLE_3V))
#define _Hw_D_SetVccOff()       WRITE_PORT_BYTE(SM_REG_CTRL_5,  0x00)

/*****/
//#define _Hw_D_SetIntMask()    WRITE_PORT_BYTE(SM_REG_INT_MASK, 0x9C)
#define _Hw_D_SetIntMask()    WRITE_PORT_BYTE(SM_REG_INT_MASK, 0x8C)
#define _Hw_D_ClrIntMask()    WRITE_PORT_BYTE(SM_REG_INT_MASK, 0x00)
#define _Hw_D_ChkIntCardChg() (READ_PORT_BYTE(SM_REG_INT_STATUS) & 0x0C)
#define _Hw_D_ClrIntCardChg() WRITE_PORT_BYTE(SM_REG_INT_STATUS, 0x0C)

#define _Hw_D_ChkIntCmdDone() (READ_PORT_BYTE(SM_REG_INT_STATUS) & 0x10)
#define _Hw_D_ClrIntCmdDone() WRITE_PORT_BYTE(SM_REG_INT_STATUS, 0x10)

// For DMA Interrupt
#define _Hw_D_SetDMAIntMask()    WRITE_PORT_BYTE(SM_DMA_INT_CTRL, 0x81)
#define _Hw_D_ClrDMAIntMask()    WRITE_PORT_BYTE(SM_DMA_INT_CTRL, 0x00)
#define _Hw_D_ChkDMAIntCardChg() (READ_PORT_BYTE(SM_DMA_INT_STATUS) & 0x01)
#define _Hw_D_ClrDMAIntCardChg() WRITE_PORT_BYTE(SM_DMA_INT_STATUS, 0x01)

#endif // already included
